<?php

/**
 * This File contains QUI\Sensor\Manager
 */

namespace QUI\Sensor;

/**
 * Sensor Manager
 *
 * @author www.pcsg.de (Henning Leutz)
 */

class Manager
{
    /**
     * Read all sensors and set the data to the database
     */
    static function read()
    {
        $list = $this->getList();

        foreach ( $list as $entry )
        {
            try
            {
                switch ( $entry['sensorType'] )
                {
                    case 'GrovePi':
                        $Sensor = new GrovePi( $entry['id'] );
                    break;
                }

                $Sensor->setPinData();

            } catch ( \QUI\Exception $Exception )
            {
                \QUI\System\Log::addError(
                    $Exception->getMessage()
                );
            }
        }
    }

    /**
     * Add a sensor to the list
     * Which sensor should be read?
     *
     * @param String $type
     * @param String $sensorPin
     * @param String $title
     */
    public function addSensor($type='GrovePi', $sensorPin, $title='')
    {
        switch ( $type )
        {
            case 'GrovePi':
                $this->_addGrovePieSensor( $sensorPin, $title );
            break;

            default:
                break;
        }
    }

    /**
     * Edit a sensor from the list
     *
     * @param unknown $sensorId
     * @param unknown $sensorType
     * @param unknown $sensorPin
     * @param string $title
     */
    public function editSensor($sensorId, $sensorType, $sensorPin, $title='')
    {
        $Sensor = $this->getSensorById( $sensorId );

        switch ( $sensorType )
        {
            case 'GrovePi':
                $this->_editGrovePieSensor( $sensorId, $sensorPin, $title );
            break;

            default:
                break;
        }
    }


    /**
     * Delete the sensor ids
     *
     * @param Array $ids - List of ids
     */
    public function deleteSensorIds($ids)
    {
        $DataBase = \QUI::getDataBase();
        $table    = \QUI::getDBTableName( 'sensors' );

        foreach ( $ids as $id )
        {
            $id = (int)$id;

            if ( $this->getSensorById( $id ) === false ) {
                return;
            }

            $DataBase->delete($table, array(
                'id' => $id
            ));
        }
    }

    /**
     * return the inserted sensor list
     *
     * @return array
     */
    public function getList()
    {
        return \QUI::getDataBase()->fetch(array(
            'from' => \QUI::getDBTableName( 'sensors' )
        ));
    }

    /**
     * Return the Sensor
     *
     * @param Integer $id - ID of the Sensor
     * @return Sensor|false
     */
    public function getSensorById($id)
    {
        $result = \QUI::getDataBase()->fetch(array(
            'from'  => \QUI::getDBTableName( 'sensors' ),
            'where' => array(
                'id' => $id
            ),
            'limit' => 1
        ));

        if ( !isset( $result[ 0 ] ) ) {
            return false;
        }

        if ( $result[ 0 ]['sensorType'] == 'GrovePi' ) {
            return new GrovePi( $result[ 0 ]['id'] );
        }

        return false;
    }

    /**
     * Add a Grove-PI Sensor
     *
     * @param string $sensorPin
     * @param string $title
     */
    protected function _addGrovePieSensor($sensorPin, $title='')
    {
        switch ( $sensorPin )
        {
            // allowed pins
            case 'A0':
            case 'A1':
            case 'A2':
            case 'D2':
            case 'D3':
            case 'D4':
            case 'D7':
            case 'D7':
            break;

            default:
                throw new \QUI\Exception(
                    \QUI::getLocale()->get(
                        'package/sensor',
                        'exception.grovpie.pin.not.supported'
                    )
                );
            break;
        }

        \QUI::getDataBase()->insert(
            \QUI::getDBTableName( 'sensors' ),
            array(
                'sensorPin'  => $sensorPin,
                'sensorType' => 'GrovePi',
                'title'      => \QUI\Utils\Security\Orthos::clear( $title )
            )
        );
    }

    /**
     *
     * @param unknown $sensorId
     * @param unknown $sensorPin
     * @param unknown $title
     */
    protected function _editGrovePieSensor($sensorId, $sensorPin, $title)
    {
        switch ( $sensorPin )
        {
            // allowed pins
            case 'A0':
            case 'A1':
            case 'A2':
            case 'D2':
            case 'D3':
            case 'D4':
            case 'D7':
            case 'D7':
            break;

            default:
                throw new \QUI\Exception(
                    \QUI::getLocale()->get(
                        'package/sensor',
                        'exception.grovpie.pin.not.supported'
                    )
                );
            break;
        }

        \QUI::getDataBase()->update(
            \QUI::getDBTableName( 'sensors' ),
            array(
                'sensorPin'  => $sensorPin,
                'sensorType' => 'GrovePi',
                'title'      => \QUI\Utils\Security\Orthos::clear( $title )
            ),
            array(
                'id' => $sensorId
            )
        );
    }
}
